@@ -97,7 +97,7 @@ iter_char_test (PangoLayout *layout)
   for (i = 0 ; i < num_chars; ++i)
     {
       gchar *char_str;
-      g_assert (iter_next_ok);
+      g_assert_true (iter_next_ok);
 
       index = pango_layout_iter_get_index (iter);
       ptr = text + index;
@@ -145,8 +145,8 @@ iter_char_test (PangoLayout *layout)
 	  verbose ("  (index_to_x ind=%d: expected x=%d, width=%d)\n",
 		   index - offset, x0, x1 - x0);
 
-	  g_assert (extents.x == x0);
-	  g_assert (extents.width == x1 - x0);
+	  g_assert_cmpint (extents.x, ==, x0);
+	  g_assert_cmpint (extents.width, ==, x1 - x0);
 	}
       else
 	{
@@ -159,7 +159,7 @@ iter_char_test (PangoLayout *layout)
 
   /* There should be one character position iterator for each character in the
    * input string */
-  g_assert (!iter_next_ok);
+  g_assert_false (iter_next_ok);
 
   pango_layout_iter_free (iter);
 }
@@ -182,7 +182,7 @@ iter_cluster_test (PangoLayout *layout)
       PangoLayoutLine *line = pango_layout_iter_get_line (iter);
 
       /* Every cluster is part of a run */
-      g_assert (pango_layout_iter_get_run (iter));
+      g_assert_true (pango_layout_iter_get_run (iter));
 
       index = pango_layout_iter_get_index (iter);
 
@@ -198,17 +198,17 @@ iter_cluster_test (PangoLayout *layout)
 
       /* All the clusters on a line should be next to each other and occupy
        * the entire line. They advance linearly from left to right */
-      g_assert (extents.width >= 0);
+      g_assert_cmpint (extents.width, >=, 0);
 
       if (last_line == line)
-	g_assert (extents.x == expected_next_x);
+	g_assert_cmpint (extents.x, ==, expected_next_x);
 
       expected_next_x = extents.x + extents.width;
 
       last_line = line;
     }
 
-  g_assert (!iter_next_ok);
+  g_assert_false (iter_next_ok);
 
   pango_layout_iter_free (iter);
 }
@@ -222,7 +222,7 @@ test_layout_iter (void)
   PangoFontDescription *font_desc;
   PangoLayout  *layout;
 
-  fontmap = pango_cairo_font_map_get_default ();
+  fontmap = pango_cairo_font_map_new ();
   context = pango_font_map_create_context (fontmap);
   font_desc = pango_font_description_from_string ("cantarell 11");
   pango_context_set_font_description (context, font_desc);
@@ -245,6 +245,7 @@ test_layout_iter (void)
   g_object_unref (layout);
   g_object_unref (context);
   pango_font_description_free (font_desc);
+  g_object_unref (fontmap);
 }
 
 static void
@@ -258,7 +259,7 @@ test_glyphitem_iter (void)
   const char *text;
   GSList *l;
 
-  fontmap = pango_cairo_font_map_get_default ();
+  fontmap = pango_cairo_font_map_new ();
   context = pango_font_map_create_context (fontmap);
   font_desc = pango_font_description_from_string ("cantarell 11");
   pango_context_set_font_description (context, font_desc);
@@ -278,7 +279,7 @@ test_glyphitem_iter (void)
     {
       PangoGlyphItemIter iter;
       gboolean have_cluster;
-
+      PangoGlyphItemIter *iter2;
 
       for (have_cluster = direction ?
 	     pango_glyph_item_iter_init_start (&iter, run, text) :
@@ -289,9 +290,18 @@ test_glyphitem_iter (void)
 	     pango_glyph_item_iter_prev_cluster (&iter))
       {
         verbose ("start index %d end index %d\n", iter.start_index, iter.end_index);
-        g_assert (iter.start_index < iter.end_index);
-        g_assert (iter.start_index + 2 >= iter.end_index);
-        g_assert (iter.start_char + 1 == iter.end_char);
+        g_assert_true (iter.start_index < iter.end_index);
+        g_assert_true (iter.start_index + 2 >= iter.end_index);
+        g_assert_true (iter.start_char + 1 == iter.end_char);
+
+        iter2 = pango_glyph_item_iter_copy (&iter);
+        g_assert_true (iter2->start_glyph == iter.start_glyph);
+        g_assert_true (iter2->start_index == iter.start_index);
+        g_assert_true (iter2->start_char == iter.start_char);
+        g_assert_true (iter2->end_glyph == iter.end_glyph);
+        g_assert_true (iter2->end_index == iter.end_index);
+        g_assert_true (iter2->end_char == iter.end_char);
+        pango_glyph_item_iter_free (iter2);
       }
     }
   }
@@ -299,6 +309,7 @@ test_glyphitem_iter (void)
   g_object_unref (layout);
   g_object_unref (context);
   pango_font_description_free (font_desc);
+  g_object_unref (fontmap);
 }
 
 int
